from typing import Optional

from huibiao_framework.backend import CommonStatusCode
from huibiao_framework.backend.exception import BasicApiInnerException
from huibiao_framework.backend.status_code import (
    BasicApiId,
    BasicApiInnerErrorCode,
    BasicStatusCode,
)
from huibiao_framework.config.config import ErrorCodeConfig
import inspect
import sys


class ApiErrorCodeManage:
    """
    构建”5aaabb“错误码，aaa标书接口id，bb表示接口内部错误码
    """

    MAX_LEN = 8

    @classmethod
    def add_service_prefix(cls, v: int):
        """
        对于失败的状态码，添加服务号码前缀
        """
        v_str = str(v)
        if len(v_str) == cls.MAX_LEN:
            return v
        if (
            v is not None
            and v != CommonStatusCode.SUCCESS.code
            and ErrorCodeConfig.SERVICE_PREFIX
        ):
            v = int(f"{ErrorCodeConfig.SERVICE_PREFIX}{v_str}")
        return v

    @classmethod
    def set_error_code_prefix_env(cls, prefix: Optional[int] = None):
        """
        服务启动时执行这个函数
        也可以启动服务时通过环境变量ERROR_CODE_SERVICE_PREFIX指定服务前缀
        如果输入的prefix code不符合规则，则忽略，保持原来的6位
        """
        if prefix and 1 <= prefix <= 99:
            ErrorCodeConfig.SERVICE_PREFIX = prefix

    @classmethod
    def gen_api_error_code_msg(
        cls,
        api_id: BasicApiId,
        api_name: str = "",
        e: Optional[BasicApiInnerException | Exception] = None,
    ):
        """
        返回6位的错误码，表示内部错误
        """

        if e is None or not isinstance(e, BasicApiInnerException):
            inner_error_code = 0
            inner_error_msg = str(e)
        else:
            inner_error_code = e.code
            inner_error_msg = e.msg

        api_error_code = cls.__gen_error_code(api_id.value, inner_error_code)
        api_error_msg = f"接口[{api_name}]内部异常,报错:{inner_error_msg}"

        return api_error_code, api_error_msg

    @classmethod
    def __gen_error_code(cls, api_id: int, inner_error_code: int):
        return 500000 + api_id * 100 + inner_error_code

    @classmethod
    def scan_module_and_summary(cls, *list_module) -> list:
        """扫描模块中的ApiId和错误码枚举类"""
        api_id_dict: dict = {}
        api_err_code_info = []

        cls_list = []
        for m in list_module:
            cls_list.extend(
                [obj for name, obj in inspect.getmembers(m) if inspect.isclass(obj)]
            )

        for obj in set(cls_list):
            # 查找继承自BasicApiId的枚举类
            if issubclass(obj, BasicApiId) and obj != BasicApiId:
                for enum_member in obj:
                    api_id_dict[enum_member.value] = enum_member.name
                    api_err_code_info.append(
                        [
                            enum_member.value,
                            cls.__gen_error_code(enum_member.value, 0),
                            "",
                        ]
                    )

            # 查找继承自BasicApiInnerErrorCode的枚举类
            elif (
                issubclass(obj, BasicApiInnerErrorCode)
                and obj != BasicApiInnerErrorCode
                and hasattr(obj, "__api_id__")
            ):
                api_id_enum: BasicApiId = getattr(obj, "__api_id__")
                for enum_member in obj:
                    api_err_code_info.append(
                        [
                            api_id_enum.value,
                            cls.__gen_error_code(api_id_enum.value, enum_member.value),
                            enum_member.msg,
                        ]
                    )

        for idx, (api_code, _, _) in enumerate(api_err_code_info):
            api_err_code_info[idx][0] = api_id_dict[api_code]

        # 排序
        api_err_code_info.sort(key=lambda x: x[1])

        return api_err_code_info

    @classmethod
    def summary_module_api_error_code_markdown(cls, *list_module):
        """
        打印模块中的ApiId和错误码枚举类
        """
        result = cls.scan_module_and_summary(*list_module)

        print("| ApiName | ErrorCode | Message |")
        print("|--------|------------|---------|")
        for api_id, api_code, msg in result:
            print(f"| {api_id} | {api_code} | {msg} |")


class BasicCodeManage:
    __DEFAULT_MODULE = sys.modules["huibiao_framework.backend.status_code"]

    @classmethod
    def scan_module_and_summary(cls, *list_module) -> list:
        """扫描模块中BasicStatusCode枚举类及其子类"""
        if cls.__DEFAULT_MODULE not in list_module:
            list_module = list_module + (cls.__DEFAULT_MODULE,)

        result = []
        cls_list = []

        for m in list_module:
            cls_list.extend(
                [obj for name, obj in inspect.getmembers(m) if inspect.isclass(obj)]
            )

        for obj in set(cls_list):
            if issubclass(obj, BasicStatusCode) and obj != BasicApiId:
                for enum_member in obj:
                    result.append([enum_member.value, enum_member.msg])

        result.sort(key=lambda x: x[0])

        return result

    @classmethod
    def summary_error_code_markdown(cls, *list_module):
        """
        打印模块中的ApiId和错误码枚举类
        """
        result = cls.scan_module_and_summary(*list_module)

        print("| Code | Message |")
        print("|--------|------------|")
        for code, msg in result:
            print(f"| {code} | {msg} |")



from typing import Optional, Dict, List, Generic, TypeVar

from pydantic import BaseModel


T = TypeVar("T")


class BidBackendBaseVo(BaseModel, Generic[T]):
    code: int
    message: str
    requestId: str
    result: T

    def is_success(self):
        return self.code == 0 and self.message == "success"


class BidBackendFileParseSubmitDto(BaseModel):
    """
    提交文件解析任务
    """

    class PdfPath(BaseModel):
        pdf_name: Optional[str] = None
        pdf_link: Optional[str] = None

    class DocxPath(BaseModel):
        docx_name: Optional[str] = None
        docx_link: Optional[str] = None

    pdf_path: PdfPath = PdfPath()
    docx_path: DocxPath = DocxPath()


class ParseResultVo(BaseModel):
    """
    获取文件解析结果
    """

    status: str
    finished: bool
    result: Optional[Dict | List] = None



import os
from typing import Optional
from urllib.parse import urlparse, unquote


class UrlUtils:
    @classmethod
    def extract_url_filename(cls, url: str) -> str:
        """
        从下载链接中获取文件名，去除其他参数
        """
        parsed = urlparse(url)
        return unquote(os.path.basename(parsed.path))

    @classmethod
    def is_http_url(cls, url: str) -> bool:
        """
        判断是否为http链接
        """
        return url.startswith("http://") or url.startswith("https://")

    @classmethod
    def transfer_url(cls, url: Optional[str]):
        """
        如果是http链接则提取出文件名
        如果是本地路径则直接返回
        """
        if url and cls.is_http_url(url):
            return cls.extract_url_filename(url)
        else:
            return url


from typing import List
from urllib.parse import urljoin

from huibiao_framework.utils.meta_class import OsAttrMeta, ConstantClass
import os


class ErrorCodeConfig:
    SERVICE_PREFIX: int = os.getenv("ERROR_CODE_SERVICE_PREFIX", "0")


class MetricsConfig(metaclass=OsAttrMeta):
    """后端服务配置"""
    IGNORE_REQUEST_OPERATION_METRICS: bool = False
    IGNORE_CLIENT_OPERATION_METRICS: bool = False


class ServiceConfig(metaclass=OsAttrMeta):
    HTTP_PORT: int = 8100
    TIMEOUT_KEEP_ALIVE: int  = 1000
    PROCESS_NUM: int = 1


class LogConfig(metaclass=OsAttrMeta):
    LOGGER_PATH: str = "/logger"
    LOG_RETENTION_DAY: int = 60
    DISABLE_LOG_PKG: str = ""
    LOG_ADD_CONTAINED_ID: bool = False
    SAVE_INFO_LEVEL: bool = False
    SAVE_DEBUG_LOG: bool = True

    @classmethod
    def get_disable_log_pkg(cls) -> List[str]:
        if cls.DISABLE_LOG_PKG:
            return cls.DISABLE_LOG_PKG.split(",")
        else:
            return []


class MinioConfig(metaclass=OsAttrMeta):
    ENDPOINT: str
    AK: str
    SK: str
    BUCKET_NAME: str = "huibiao"
    OSS_SECURE: bool = False


class LlmModelNameConstant(ConstantClass):
    HuiZeQwen32bAwq = "HuiZeQwen32bAwq"
    MindileDs32bAwq = "MindileDs32bAwq"


class ModelConfig(metaclass=OsAttrMeta):
    """
    大语言模型的参数
    HuiZeQwen32bQwq
    内蒙环境 182.42.234.26  http://vllm-qwen-32b.model.hhht.ctcdn.cn:9080/common/query
    MindileDs32bAwq
    上海910b智能体环境 61.172.167.158 http://mindie-ds-32b-awq.office.sh.ctyun.cn:9080/v1/chat/completions
    """
    LLM_MODEL_TYPE: str = LlmModelNameConstant.HuiZeQwen32bAwq
    REQUEST_URL: str = "http://vllm-qwen-32b.model.hhht.ctcdn.cn:9080/common/query"
    OPEN_AI_MODEL_TYPE: str = "huize"
    VLLM_BEAR_TOKEN: str = ""

class ClientConfig(metaclass=OsAttrMeta):
    SEARCH_URL: str = "http://huibiao-omni-retrieve-engine-dev.zz.ctyun.cn:9080"
    EMBED_URL: str = "http://dmx.model.zz.ctcdn.cn:9080/bas-bge-large-zhengwu-0329/embedding"
    CHUNK_URL: str = (
        "http://dmx.model.dev.zz.ctcdn.cn:9080/text-content-chunk-tools-dev/chunk"
    )
    BID_BACKEND_URL: str = "http://huize-bid-backend-dev.ctyun.cn:9080"


class OcrDocumentParseClientConfig:
    """
    tender-document-parser的配置
    """
    TENDER_DOCUMENT_PARSER_URL = os.getenv(
        "TENDER_DOCUMENT_PARSER_URL",
        "http://tender-document-parser-dev.hhht.ctcdn.cn:9080",
    )

    # 各服务地址配置 - 支持独立指定不同服务地址
    IMAGE_OCR_TYY_URL = os.getenv(
        "IMAGE_OCR_TYY_URL", urljoin(TENDER_DOCUMENT_PARSER_URL, "image_ocr")
    )
    LAYOUT_DETECTION_TYY_URL = os.getenv(
        "LAYOUT_DETECTION_TYY_URL",
        urljoin(TENDER_DOCUMENT_PARSER_URL, "image_layout"),
    )
    DOCUMENT_PARSER_TYY_URL = os.getenv(
        "DOCUMENT_PARSER_TYY_URL", urljoin(TENDER_DOCUMENT_PARSER_URL, "upload")
    )
    CONVERT_TO_PDF_URL = os.getenv(
        "CONVERT_TO_PDF_URL", urljoin(TENDER_DOCUMENT_PARSER_URL, "convert")
    )
    DOCUMENT_PARSER_TYY_ASYNC_UPLOAD_URL = os.getenv(
        "DOCUMENT_PARSER_TYY_ASYNC_UPLOAD_URL",
        urljoin(TENDER_DOCUMENT_PARSER_URL, "async_upload"),
    )
    DOCUMENT_PARSER_TYY_ASYNC_PULL_URL = os.getenv(
        "DOCUMENT_PARSER_TYY_ASYNC_PULL_URL",
        urljoin(TENDER_DOCUMENT_PARSER_URL, "async_pull"),
    )


import os
import traceback
from typing import List

from loguru import logger

from huibiao_framework.utils.file_info_utils import DirectoryInfo, LocalAbstractFileInfo
from datetime import datetime, timedelta


class LocalOutdatedDirCleaner:
    """
    指定一个父目录，清理该目录下的过期目录，可以指定一个过期时间
    """

    def __init__(self, path: str, retention_day: int = 30, theme: str = ""):
        self.path = path
        self.retention_day = retention_day
        self.theme = theme
        self.current_time: datetime = datetime.now()
        self.threshold_time: datetime = self.current_time - timedelta(days=self.retention_day)

    def do_clean(self):
        """
        开始清除
        """
        outdated_dirs = self.search_unused_directory()
        logger.info(f"{self.theme} 开始清理缓存目录，共{len(outdated_dirs)}个目录")
        failed = self.batch_delete_file(outdated_dirs)
        logger.info(f"{self.theme} 缓存目录清理完成，失败数{len(failed)}/{len(outdated_dirs)}")

    def is_outdated(self, directory: DirectoryInfo) -> bool:
        """
        可以通过覆盖此类，实现定制功能
        """
        return (
            directory.access_time < self.threshold_time
            and directory.modify_time < self.threshold_time
            and directory.modify_time < self.threshold_time
        )

    def search_unused_directory(self) -> List[DirectoryInfo]:
        """
        在指定目录下搜索长时间不用的文件夹
        """

        if not os.path.exists(self.path):
            return []

        dir_list = [os.path.join(self.path, d) for d in os.listdir(self.path)]

        dir_list = [
            DirectoryInfo(d)
            for d in dir_list
            if os.path.isdir(d)
        ]

        result = [d for d in dir_list if self.is_outdated(d)]

        logger.debug(f"path={self.path}, threshold_time={self.threshold_time}, retention_day={self.retention_day}, found outdated dir, {len(result)}")

        return result

    @classmethod
    def batch_delete_file(cls, files: List[LocalAbstractFileInfo]) -> List[LocalAbstractFileInfo]:
        """
        批量删除文件
        返回删除失败的文件
        """
        if not files:
            return []
        failed: List[LocalAbstractFileInfo] = []
        num = len(files)
        for idx, f in enumerate(files):
            try:
                f.delete()
                logger.debug(f"[{idx + 1}]/[{num}] delete file {f.path} success")
            except Exception as e:
                logger.error(f"[{idx + 1}]/[{num}] delete file {f.path} failed, {e}, {traceback.format_exc()}")
                failed.append(f)
        return failed


import aiohttp
import time
from loguru import logger
import asyncio

from huibiao_framework.client.data_model.bid_backend import (
    BidBackendFileParseSubmitDto,
    ParseResultVo,
    BidBackendBaseVo,
)
from huibiao_framework.config import ClientConfig


class BidBackendFileParseClient:
    """
    调用bid-backend服务的文件解析接口，并获取结果，保存到本地
    """

    def __init__(self, host: str = None):
        self.host = host if host else ClientConfig.BID_BACKEND_URL

    async def submit(
        self, session: aiohttp.ClientSession, dto: BidBackendFileParseSubmitDto
    ) -> int:
        """
        提交文件解析任务。然会project_id
        """
        async with session.post(
            f"{self.host}/project/fileParse", json=dto.model_dump()
        ) as response:
            response.raise_for_status()
            resp_json = await response.json()
            vo = BidBackendBaseVo[int](**resp_json)
            assert vo.is_success(), f"Reqid:{vo.requestId}, 提交任务失败，{vo.message}"
            return vo.result

    async def query_result(
        self, session: aiohttp.ClientSession, project_id: str | int
    ) -> ParseResultVo:
        """
        获取文件解析结果
        """
        async with session.get(
            f"{self.host}/projects/{str(project_id)}/fileParseResult"
        ) as response:
            response.raise_for_status()
            resp_json = await response.json()
            vo = BidBackendBaseVo[ParseResultVo](**resp_json)
            return vo.result

    async def submit_and_query(
        self,
        dto: BidBackendFileParseSubmitDto,
        *,
        session: aiohttp.ClientSession,
        sleep_sec: int = 10,
        attempts: int = 30,
    ):
        """
        提交文件解析任务，轮询结果，并返回下载链接
        """
        project_id: int = await self.submit(session=session, dto=dto)
        start_time = time.perf_counter()
        for i in range(attempts):
            r = await self.query_result(project_id=project_id, session=session)
            if r.result is not None and r.finished:
                logger.info(
                    f"Project_id:{project_id}, 第{i + 1}次获取结果,成功，耗时：{time.perf_counter() - start_time}s"
                )
                return r.result
            else:
                logger.debug(
                    f"Project_id:{project_id}, 第{i + 1}次获取结果,statu={r.status},finished={r.finished},耗时：{time.perf_counter() - start_time}s"
                )
                await asyncio.sleep(sleep_sec)
        logger.info(f"Reqid:{project_id}, 获取结果超时")
        return None



from pydantic import BaseModel, Field, model_validator
from typing import Generic, TypeVar, Optional

from huibiao_framework.execption.model import (
    HuiZeModelResponseCodeError,
    HuiZeModelResponseFormatError,
)

T = TypeVar("T")


class ModelBaseRespVo(BaseModel, Generic[T]):
    # 响应状态码，0通常表示成功
    code: int = Field(..., description="响应状态码，0表示成功")

    # 响应消息
    message: str = Field("", description="响应状态描述信息")

    # 分析结果数据（泛型类型，可动态指定）
    result: Optional[T] = Field(None, description="分析结果数据，类型由泛型参数指定")

    @model_validator(mode="after")
    def check_result_consistent_with_code(self) -> "ModelBaseRespVo":
        if self.code is None or self.code != 0:
            # 校验 code 合法性（通常 0 表示成功，非 0 表示错误）
            raise HuiZeModelResponseCodeError(self.code, self.message)
        if self.code == 0 and self.result is None:
            raise HuiZeModelResponseFormatError("Field 'result' is empty!")
        return self



from loguru import logger
from fastapi.exceptions import StarletteHTTPException
from fastapi.responses import JSONResponse
from starlette.requests import Request

from huibiao_framework.backend import BaseRespVo, CommonStatusCode
from huibiao_framework.backend.utils import extract_url


def http_exception_interceptor(request: Request, exc: StarletteHTTPException):
    url = extract_url(request)

    reqid: str = request.headers.get("x-request-id", None)

    logger.info(f"req_id: {url} | 错误码: {exc.status_code} | 错误信息: {exc.detail}")
    if exc.status_code == 405:
        vo = BaseRespVo.from_status_code(CommonStatusCode.ERROR_REQ_METHOD, url=url)
    elif exc.status_code == 404:
        vo = BaseRespVo.from_status_code(CommonStatusCode.ERROR_REQ_URL)
    else:
        vo = BaseRespVo.from_status_code(
            CommonStatusCode.UNKNOWN_ERROR, msg=f"{exc.status_code},{exc.detail}"
        )

    return JSONResponse(content=vo.set_request_id(reqid).model_dump(), status_code=200)



from .huize_qwen32b_awq_client import HuiZeQwen32bQwqClient
from .abstrast_llm_client import AbstractLlmModelClient
from .llm_client import LlmModelClient

__all__ = [
    "HuiZeQwen32bQwqClient",
    "LlmModelClient",
    "AbstractLlmModelClient",
]



from typing import Optional, List, Callable, Coroutine, Any

from fastapi import FastAPI, APIRouter
from fastapi.exceptions import RequestValidationError, StarletteHTTPException
from fastapi.middleware.cors import CORSMiddleware
from contextlib import asynccontextmanager
from loguru import logger

from huibiao_framework.backend import (
    ApiErrorCodeManage,
    PrometheusTimeCostMetricSetting,
)
from huibiao_framework.backend.intercept import (
    http_exception_interceptor,
    validate_interceptor,
)
from huibiao_framework.backend.log_utils import ServiceLoggerManager
from huibiao_framework.backend.metrics import PrometheusContext
from huibiao_framework.backend.metrics.collector import MetricsScheduleMonitor


async def default_life_span_start_event():
    logger.info("用户未设置开始事件")


async def default_life_span_end_event():
    logger.info("用户未设置结束事件")


class HuibiaoDefaultAppGenerator:
    def __init__(
        self,
        *,
        app_error_code_prefix: int,
        service_name: str,
        life_span_start_event: Optional[Callable[[], Coroutine[Any, Any, None]]] = None,
        life_span_end_event: Optional[Callable[[], Coroutine[Any, Any, None]]] = None,
        api_routers: List[APIRouter] = None,
    ):
        # 注册8位错误码前缀
        ApiErrorCodeManage.set_error_code_prefix_env(app_error_code_prefix)
        self.service_name = service_name
        self.api_routers = api_routers or []

        self.__app: Optional[FastAPI] = FastAPI()
        self.__life_span_start = life_span_start_event or default_life_span_start_event
        self.__life_span_end = life_span_end_event or default_life_span_end_event

    def generator(self):
        self.init_app()
        self.add_prometheus_endpoint()
        self.add_router(*self.api_routers)
        self.add_default_cost_middleware()
        self.add_generic_exception_handler()
        self.add_http_exception_handler()
        return self.app

    def get_app(self):
        return self.__app

    @property
    def app(self):
        return self.__app

    def init_app(self, **kwargs):
        self.__app = FastAPI(lifespan=self.gen_life_span(), **kwargs)

    def add_prometheus_endpoint(self) -> "HuibiaoDefaultAppGenerator":
        PrometheusContext.mount_prometheus_endpoint(self.__app)
        return self

    def add_router(self, *api_router: APIRouter) -> "HuibiaoDefaultAppGenerator":
        for rt in api_router:
            self.__app.include_router(rt)
        return self

    def add_default_cost_middleware(self) -> "HuibiaoDefaultAppGenerator":
        self.__app.add_middleware(
            CORSMiddleware,
            allow_origins=["*"],
            allow_credentials=True,
            allow_methods=["*"],
            allow_headers=["*"],
        )
        return self

    def add_generic_exception_handler(self) -> "HuibiaoDefaultAppGenerator":
        """
        @app.exception_handler(RequestValidationError)
        async def generic_exception_handler(request: Request, exc: RequestValidationError):
            return validate_interceptor(request=request, exc=exc)
        """
        self.__app.add_exception_handler(RequestValidationError, validate_interceptor)
        return self

    def add_http_exception_handler(self) -> "HuibiaoDefaultAppGenerator":
        """
        @app.exception_handler(StarletteHTTPException)
        async def http_exception_handler(req: Request, exc: StarletteHTTPException):
            return http_exception_interceptor(request=req, exc=exc)
        """
        self.__app.add_exception_handler(
            StarletteHTTPException, http_exception_interceptor
        )
        return self

    def gen_life_span(self):
        @asynccontextmanager
        async def lifespan(app: FastAPI):
            logger.info("Start app")
            # 多worker时，worker进程的日志设置
            ServiceLoggerManager.setup_log(self.service_name)
            # 启动系统指标、gc指标的定时采集写入以供多进程时的汇总
            await MetricsScheduleMonitor().run_monitor()
            # @timing_and_exception_handler的初始化
            PrometheusTimeCostMetricSetting.initialize()

            # 执行用户提供的开始事件
            await self.__life_span_start()

            yield

            # 执行用户提供的结束事件
            await self.__life_span_end()

            # 关闭线程池
            logger.info("Shut down app")

        return lifespan



from datetime import timedelta
import functools
import os.path
from abc import ABC, abstractmethod
from typing import Generic, TypeVar

import aiohttp
from loguru import logger

from huibiao_framework.client import MinIOClient
from huibiao_framework.config import MinioConfig


def with_oss_client(func):
    @functools.wraps(func)
    async def wrapper(self, client: MinIOClient = None, *args, **kwargs):
        self.valid_oss_param()

        if client:
            return await func(self, client, *args, **kwargs)
        else:
            async with aiohttp.ClientSession() as session:
                client = MinIOClient(session=session)
                return await func(self, client, *args, **kwargs)

    return wrapper


T = TypeVar("T")


class AbstractAsyncFile(ABC, Generic[T]):
    """
    临时文件管理器
    """

    def __init__(
        self,
        *,
        local_path: str = None,
        wget_link: str = None,
        oss_key=None,
        oss_bucket: str = MinioConfig.BUCKET_NAME,
    ):
        self.local_path = local_path
        if local_path and os.path.dirname(local_path):
                os.makedirs(os.path.dirname(local_path), exist_ok=True)

        self.wget_link = wget_link
        self.oss_bucket = oss_bucket
        self.oss_key = oss_key
        self.__data: T = None

    def set_data(self, data):
        self.__data = data

    @property
    def data(self) -> T:
        return self.__data

    def exist(self) -> bool:
        return os.path.exists(self.local_path)

    @abstractmethod
    async def load(self) -> T:
        pass

    @abstractmethod
    async def save(self):
        pass

    @abstractmethod
    async def wget(self, session: aiohttp.ClientSession = None) -> T:
        """下载数据，但是不保存"""
        pass


    def delete(self):
        """
        删除本地文件
        """
        if self.local_path and os.path.exists(self.local_path):
            os.remove(self.local_path)
            logger.debug(f"file delete success: {self.local_path}")

    # region oss操作
    def valid_oss_param(self):
        assert self.local_path, "请设置文件路径"
        assert self.oss_key, "请设置文件oss_key"
        assert self.oss_bucket, "请设置文件oss_bucket"

    @with_oss_client
    async def upload_oss(self, client: MinIOClient = None):
        """上传到oss"""
        self.valid_oss_param()
        await client.upload_file(
            bucket_name=self.oss_bucket,
            object_name=self.oss_key,
            file_path=self.local_path,
        )

    @with_oss_client
    async def download_oss(self, client: MinIOClient = None):
        """从oss下载到本地"""
        self.valid_oss_param()
        await client.download_file(
            bucket_name=self.oss_bucket,
            object_name=self.oss_key,
            file_path=self.local_path,
        )

    @with_oss_client
    async def gen_oss_url(self, client: MinIOClient = None, expires = timedelta(days=1)):
        """生成oss链接"""
        self.valid_oss_param()
        return await client.get_object_url(
            bucket_name=self.oss_bucket,
            object_name=self.oss_key,
            expires=expires
        )

    # endregion
    async def basic_wget(self, session: aiohttp.ClientSession = None) -> bytes:
        """下载json，保存到本地"""
        assert self.wget_link, "请设置文件的下载链接"

        async def fun(_session):
            try:
                async with _session.get(self.wget_link) as response:
                    response.raise_for_status()
                    data: bytes = await response.read()
                    logger.info(f"success wget {self.wget_link}")
                    return data
            except Exception as e:
                logger.error(f"下载文件失败 {self.wget_link}, 原因 {str(e)}")
                raise e

        if session is not None:
            return await fun(session)
        else:
            async with aiohttp.ClientSession() as session:
                return await fun(session)



from .collector import SystemMetricsMonitor, AbstractMetricCollector
from .request_metrics import (
    RequestOperationMetrics,
    PrometheusApiMetrics,
    PrometheusTimeCostMetricSetting,
)
from .http_metrics import RequestMonitorMiddleware
from .prometheus_context import PrometheusContext


__all__ = [
    "SystemMetricsMonitor",
    "RequestOperationMetrics",
    "RequestMonitorMiddleware",
    "PrometheusContext",
    "AbstractMetricCollector",
    "PrometheusApiMetrics",
    "PrometheusTimeCostMetricSetting",
]



import os
import shutil
from abc import ABC, abstractmethod
from datetime import datetime
from typing import Optional


class LocalAbstractFileInfo(ABC):
    def __init__(self, path: str):
        self.path: str = path
        self.create_time: Optional[datetime] = None
        self.modify_time: Optional[datetime] = None
        self.access_time: Optional[datetime] = None
        self.init()

    def exists(self) -> bool:
        return os.path.exists(self.path)

    def init(self) -> 'LocalAbstractFileInfo':
        if self.exists():
            self.create_time = datetime.fromtimestamp(os.path.getctime(self.path))
            self.modify_time = datetime.fromtimestamp(os.path.getmtime(self.path))
            self.access_time = datetime.fromtimestamp(os.path.getatime(self.path))
        return self

    @abstractmethod
    def size(self):
        pass

    @abstractmethod
    def delete(self):
        pass

    def __repr__(self):
        return f"FileInfo(path={self.path})"


class FileInfo(LocalAbstractFileInfo):
    def __init__(self, path: str):
        super().__init__(path)

    @abstractmethod
    def size(self):
        return os.path.getsize(self.path)

    @abstractmethod
    def delete(self):
        if self.exists():
            os.remove(self.path)


class DirectoryInfo(LocalAbstractFileInfo):
    def __init__(self, path: str):
        super().__init__(path)

    def delete(self):
        """
        删除文件夹
        """
        if self.exists():
            shutil.rmtree(self.path)

    def size(self):
        """
        获取文件夹大小，待实现
        """
        pass


import asyncio
from typing import List

from huibiao_framework.utils.file_manage import JsonAsyncFile
from huibiao_framework.utils.url_utils import UrlUtils


async def get_local_or_http_json(url_or_path: str) -> JsonAsyncFile:
    """
    如果传入http下载路径，则只会下载到内存中，但不会落盘
    如果传入本地路径，则直接从本地加载json
    :param url_or_path:
    :return:
    """
    async_file: JsonAsyncFile
    if UrlUtils.is_http_url(url_or_path):
        async_file = JsonAsyncFile(
            wget_link=url_or_path,
        )
        await async_file.wget()
    else:
        async_file = JsonAsyncFile(local_path=url_or_path)
        await async_file.load()

    return async_file


async def get_local_or_http_json_batch(*urls_or_paths: str) -> List[JsonAsyncFile]:
    """
    批量获取json
    """
    return await asyncio.gather(
        *[get_local_or_http_json(url_or_path) for url_or_path in urls_or_paths]
    )



import time
from abc import abstractmethod, ABC
from typing import Optional, TypeVar, Union, List
import aiohttp

from huibiao_framework.backend.metrics import PrometheusTimeCostMetricSetting
from huibiao_framework.config import ModelConfig
from loguru import logger
from pydantic import BaseModel


T = TypeVar("T", bound=BaseModel)
V = TypeVar("V", bound=BaseModel)


class AbstractLlmModelClient(ABC):
    def __init__(
        self,
        client_name: str,
        *,
        session: Optional[aiohttp.ClientSession] = None,
        url: str = None,
    ):
        self.__session = session
        self.__name = client_name
        assert self.__session is not None, "会话不能为空"
        self.__url = url if url else ModelConfig.REQUEST_URL

        self.headers = {"Content-Type": "application/json"}

        if ModelConfig.VLLM_BEAR_TOKEN:
            self.headers["Authorization"] = f"Bearer {ModelConfig.VLLM_BEAR_TOKEN}"

    @property
    def url(self):
        return self.__url

    @PrometheusTimeCostMetricSetting.client_metrics_handler("LLmClient")
    async def post(self, dto: T, session_id: str = "") -> dict:
        session_tag = self.session_tag(session_id)
        logger.debug(f"{session_tag},contentLien={self.cal_content_len(dto)}")
        start_time = time.perf_counter()
        try:
            async with self.session.post(
                self.url, json=dto.model_dump(exclude_none=True), headers=self.headers
            ) as resp:
                sp_time = time.perf_counter() - start_time
                logger.debug(f"{session_tag},resp-{resp.status},cost {sp_time:.2f}s")
                resp.raise_for_status()
                response_data = await resp.json()
                return response_data
        except Exception as e:
            logger.error(f"{session_tag} failed, {str(e)}")
            raise e

    @abstractmethod
    def cal_content_len(self, dto: T) -> Union[int, List[int]]:
        return [len(i.content) for i in dto.messages]

    @abstractmethod
    async def query(self, dto: T, session_id: str = "") -> V:
        pass

    @abstractmethod
    async def simple_query(self, content, session_id: str = "", **kwargs) -> str:
        """
        简单的一次性问答
        LLMException
        """
        pass

    @property
    def session(self):
        return self.__session

    @property
    def client_name(self) -> Optional[str]:
        return self.__name

    def session_tag(self, session_id: str) -> str:
        session_id_suffix = f"[{session_id}]" if session_id else ""
        return f"[{self.client_name}]{session_id_suffix}"



from typing import Optional

import aiohttp

from huibiao_framework.client.llm.huize_qwen32b_awq_client import HuiZeQwen32bQwqClient
from huibiao_framework.client.llm.abstrast_llm_client import AbstractLlmModelClient
from huibiao_framework.client.llm.mindie_ds_32b_awq import MindleDs32bAwqClient
from huibiao_framework.config.config import ModelConfig, LlmModelNameConstant
from huibiao_framework.execption.model import LLMException
from loguru import logger


class LlmModelClient:
    __MODEL_REGISTRY = {
        LlmModelNameConstant.HuiZeQwen32bAwq: HuiZeQwen32bQwqClient,
        LlmModelNameConstant.MindileDs32bAwq: MindleDs32bAwqClient,
    }

    def __init__(self, session: aiohttp.ClientSession, model: str = None):
        self.model_name: str = model or ModelConfig.LLM_MODEL_TYPE
        self.__model: Optional[AbstractLlmModelClient] = None

        if self.model_name in self.__MODEL_REGISTRY:
            self.__model = self.__MODEL_REGISTRY[self.model_name](session)
        else:
            logger.error(f"未注册的模型类型: {self.model_name}")
            raise LLMException("错误的llm模型类型")

    async def simple_query(self, content, session_id: str = "", **kwargs) -> str:
        return await self.__model.simple_query(content, session_id, **kwargs)

    @classmethod
    def register_model(
        cls, model_name: str, model_client: type(AbstractLlmModelClient)
    ):
        """
        注册一个自定义的模型，实现AbstractLlmModelClient类
        """
        logger.info(f"注册模型: {model_name}")
        if model_name in cls.__MODEL_REGISTRY:
            logger.warning(f"已存在模型: {model_name},请勿重新注册")
        else:
            cls.__MODEL_REGISTRY[model_name] = model_client

    @staticmethod
    def register_model_decorator(model_name):
        """
        注册LLM模型的装饰器
        @LlmModelClient.register_model_decorator({模型名})
        """

        def decorator(cls):
            # 注册模型到客户端
            LlmModelClient.register_model(model_name, cls)
            return cls

        return decorator



import time
import traceback
from functools import wraps
from typing import Callable, Awaitable

from loguru import logger

from huibiao_framework.backend import (
    BaseRespVo,
    BasicCommonException,
    BasicApiId,
    CommonStatusCode,
)
from huibiao_framework.backend.metrics.request_metrics import (
    PrometheusTimeCostMetricSetting,
)
from huibiao_framework.backend.vo import ApiErrorCodeManage


def timing_and_exception_handler(func=None, *, api_id: BasicApiId, api_name: str = ""):
    """
    装饰器：用于统计函数执行时间并捕获异常
    函数中需要包含参数reqid或者request_id
    """

    def decorator(
        func: Callable[..., Awaitable[BaseRespVo]],
    ) -> Callable[..., Awaitable[BaseRespVo]]:
        @wraps(func)
        async def wrapper(*args, **kwargs):
            start_time = time.perf_counter()

            request_id = kwargs.get("request_id", None) or kwargs.get("reqid", None)
            _name = func.__name__ or api_name

            try:
                # 执行原函数
                logger.info(f"ReqId: {request_id} | Function: {_name} | Start")
                result: BaseRespVo = await func(*args, **kwargs)
                # 计算耗时
                elapsed_time = time.perf_counter() - start_time
                logger.info(
                    f"ReqId: {request_id} | Function: {_name} | COST:{elapsed_time:.4f}s"
                )
                PrometheusTimeCostMetricSetting.api_metrics_instance().add(
                    _name, elapsed_time
                )
                return result.set_request_id(request_id)
            except Exception as e:
                # 计算耗时
                elapsed_time = time.perf_counter() - start_time
                PrometheusTimeCostMetricSetting.api_metrics_instance().add_error(_name)
                # 记录异常信息和完整堆栈
                logger.error(
                    f"ReqId: {request_id} | Function: {_name} | COST:{elapsed_time:.4f}s | Exception: {str(e)}\n"
                    f"Traceback:\n{traceback.format_exc()}"
                )
                if isinstance(e, BasicCommonException):
                    vo = BaseRespVo(
                        code=e.code, message=f"接口[{_name}]报错:{e.msg}", result=None
                    )
                else:
                    api_error_code, api_error_msg = (
                        ApiErrorCodeManage.gen_api_error_code_msg(api_id, _name, e)
                    )
                    vo = BaseRespVo(
                        code=api_error_code,
                        message=api_error_msg,
                        result=None,
                    )
                vo.set_request_id(request_id)
                return vo

        return wrapper

    return decorator if func is None else decorator(func)


def basic_exception_handler(func=None, *, api_name: str = ""):
    """
    装饰器：用于统计函数执行时间并捕获异常
    """

    def decorator(
        func: Callable[..., Awaitable[BaseRespVo]],
    ) -> Callable[..., Awaitable[BaseRespVo]]:
        @wraps(func)
        async def wrapper(*args, **kwargs):
            _name = func.__name__ or api_name
            try:
                result: BaseRespVo = await func(*args, **kwargs)
                return result
            except Exception as e:
                logger.error(
                    f"Function: {_name}| Exception: {str(e)} Traceback:\n{traceback.format_exc()}"
                )
                if isinstance(e, BasicCommonException):
                    vo = BaseRespVo(
                        code=e.code, message=f"接口[{_name}]报错:{e.msg}", result=None
                    )
                else:
                    vo = BaseRespVo(
                        code=CommonStatusCode.UNKNOWN_ERROR.code,
                        message=f"接口[{_name}]报错:{str(e)}",
                        result=None,
                    )
                return vo

        return wrapper

    return decorator if func is None else decorator(func)
